import check from 'check-types';

function ipv4(v) {
  if (!check.string(v)) {
    return new Error('not a string');
  }

  const segments = v.split('.');
  if (segments.length !== 4) {
    return new Error('not a valid IPv4');
  }

  return segments
    .map(segmentValue => parseInt(segmentValue, 10))
    .every(segmentValue =>
      check.integer(segmentValue) && check.inRange(segmentValue, 0x00, 0xff));
}

function port(v) {
  return check.integer(v) &&
    check.inRange(v, 0x0001, 0xffff);
}

function positiveInteger(v) {
  return check.integer(v) && check.positive(v);
}

export default {
  ...check,
  ipv4,
  port,
  positiveInteger,
};
