import obop from 'obop';

import check from '../check';
import root from '../root';

const reduxGenerator = root.sub('serverlist');

export const initialState = {
  servers: [],
};

const myRedux = reduxGenerator.all({
  initialState,

  actions: {
    add: {
      params: {
        ip: check.ipv4, // server ip
        ping: check.number, // server ping

        name: check.string, // server name
        port: check.port, // server port
        hostPlayer: check.string, // hosting player
        sprintEnabled: v => check.includes(['0', '1'], v),
        sprintUnlimitedEnabled: v => check.includes(['0', '1'], v),
        dualWielding: v => check.includes(['0', '1'], v),
        assassinationEnabled: v => check.includes(['0', '1'], v),
        votingEnabled: check.boolean,
        teams: check.boolean,
        map: check.string,
        mapFile: check.string,
        variant: check.maybe.string,
        variantType: check.maybe.string,
        teamScores: check.maybe.array.of.number,
        status: check.string,
        numPlayers: v => check.integer(v) && (check.zero(v) || check.positive(v)),
        mods: check.array, // TODO - fine tune this
        maxPlayers: v => check.integer(v) && (check.zero(v) || check.positive(v)),
        passworded: check.boolean,
        isDedicated: check.boolean,
        gameVersion: check.string,
        eldewritoVersion: check.maybe.string,
        xnkid: check.maybe.string,
        xnaddr: check.maybe.string,
        players: v => check.maybe.array.of.like(v, {
          name: '',
          serviceTag: '',
          team: 0,
          uid: '',
          primaryColor: '#000000',
          isAlive: true,
          score: 0,
          kills: 0,
          assists: 0,
          deaths: 0,
          betrayals: 0,
          timeSpentAlive: 0,
          suicides: 0,
          bestStreak: 0,
        }),
      },

      reducer(current, payload) {
        // do we have this server on the list already?
        if (current.servers.find(server =>
          server.ip === payload.ip
            && server.port === payload.port)) {
          // skip this server
          return current;
        }

        // add server
        return {
          ...current,
          servers: [
            ...current.servers,
            payload,
          ],
        };
      },
    },

    addMultiple: {
      params: {
        servers: v => !!v, // TODO
      },
      reducer(current, { servers }) {
        return { ...current,
          servers: servers.reduce((currentServers, server) => {
          // is this an actual new server?
            if (!currentServers.find(registeredServer =>
              server.ip === registeredServer.ip
              && server.port === registeredServer.port)) {
              return [...currentServers, server];
            }

            // server is not new, return old list
            return currentServers;
          }, current.servers),
        };
      },
    },

    reset: {
      reducer: () => initialState,
    },
  },

  // selectors are all function(state, ...parameters)
  selectors: {
    /*
      Example:
      getServers(state, {
          name: {
          $in: ["substring"],
      }
      gameType: "slayer",
    })
    */
    getServers({ servers }, filter, { order } = {}) {
      let finalServers = servers;
      if (filter) {
        finalServers = obop.where(finalServers, filter);
      }
      if (order) {
        finalServers = obop.order(finalServers, order);
      }
      return finalServers;
    },
  },
});

export const {
  actions,
  actionTypes,
  reducer,
  selectors,
} = myRedux;
